package com.example.libmedia.demo.video;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.support.v4.app.ListFragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import java.util.List;

import fr.maxcom.os.storage.Volume;
import fr.maxcom.os.storage.VolumeManager;

public class StorageSettingsFragment extends ListFragment implements OnClickListener {
	private static final int STORAGE_ACCESS_REQUEST_CODE = 99;  // any value of your choice

	private ItemAdapter<Volume> mAdapter;
	private Intent mRequestStorageAccessIntent;
	private VolumeManager mVolumeManager;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		mVolumeManager = new VolumeManager();
		// A null means that Storage Access Framework is not activated.
		mRequestStorageAccessIntent = VolumeManager.getStorageAccessIntent();
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
		final View v = inflater.inflate(R.layout.fragment_storage, container, false);
		final Button btn = (Button) v.findViewById(R.id.btnAddVolume);
		if (mRequestStorageAccessIntent != null) {
			btn.setOnClickListener(this);
		} else {
			// The button is not applicable in this running context, hide it.
			btn.setVisibility(View.GONE);
		}
		final List<Volume> volumes = mVolumeManager.getVolumes();
		mAdapter = new ItemAdapter<Volume>(getActivity(), R.layout.list_volume, volumes);
		setListAdapter(mAdapter);
		return v;
	}

	//----- OnClickListener Interface -----
	@Override
	public void onClick(View v) {
		switch (v.getId()) {
		case R.id.btnAddVolume:
			requestStorageAccess();
			break;
		}
	}

	private class ItemAdapter<T extends Volume> extends ArrayAdapter<T> implements OnClickListener {
		private LayoutInflater mInflater;
		private int mResource;

		public ItemAdapter(Context context, int resource, List<T> objects) {
			super(context, resource, objects);
			init(context, resource);
		}

		private void init(Context context, int resource) {
			mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			mResource = resource;
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			View view;
			if (convertView == null) {
				view = mInflater.inflate(mResource, parent, false);
				view.findViewById(R.id.btnEditLabel).setOnClickListener(this);
				view.findViewById(R.id.btnRemoveLabel).setOnClickListener(this);
				view.findViewById(R.id.btnForget).setOnClickListener(this);
			} else {
				view = convertView;
			}
			T item = getItem(position);
			bindView(view, item);
			return view;
		}

		private void bindView(View view, T item) {
			final String label = item.getLabel();
			final String name = item.getName();
			final StringBuilder sb = new StringBuilder().append(label);
			if (name != null && !name.equals(label)) sb.append(" (").append(name).append(")");
			((TextView) view.findViewById(R.id.txLabel)).setText(sb.toString());
			view.findViewById(R.id.btnRemoveLabel)
					.setVisibility(item.hasLabelFile() ? View.VISIBLE : View.GONE);
			view.findViewById(R.id.btnForget)
					.setVisibility(item.isForgettable() ? View.VISIBLE : View.GONE);
		}

		//----- onClick Helpers -----
		private Volume getTargetVolume(View v) {
			final int position = getListView().getPositionForView(v);
			if (position != AdapterView.INVALID_POSITION) {
				return mAdapter.getItem(position);
			}
			return null;
		}

		//----- OnClickListener Interface -----
		@Override
		public void onClick(View v) {
			final Volume volume = getTargetVolume(v);
			if (volume != null) {
				switch (v.getId()) {
				case R.id.btnEditLabel:
					final DialogFragment f = EditLabelDialogFragment.newInstance(mAdapter.getPosition(volume), volume.getLabel());
					f.setTargetFragment(StorageSettingsFragment.this, 0);
					f.show(getFragmentManager(), "EditLabelDialogFragment");
					break;
				case R.id.btnRemoveLabel:
					if (volume.deleteLabelFile()) refreshVolumes();
					break;
				case R.id.btnForget:
					if (volume.forget()) renewVolumes();
					break;
				}
			}
		}
	}

	//----- Processors -----
	private void requestStorageAccess() {
		startActivityForResult(mRequestStorageAccessIntent, STORAGE_ACCESS_REQUEST_CODE);
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent resultData) {
		if (requestCode == STORAGE_ACCESS_REQUEST_CODE && resultCode == Activity.RESULT_OK) {
			// Forward the result, to persist the selection.
			VolumeManager.onStorageAccessResult(resultData);
			// The list needs to be fully refreshed.
			renewVolumes();
		}
	}

	private static class EditLabelDialogFragment extends DialogFragment {
		static EditLabelDialogFragment newInstance(int position, String value) {
			final EditLabelDialogFragment f = new EditLabelDialogFragment();
			final Bundle args = new Bundle();
			args.putInt("position", position);
			args.putString("value", value);
			f.setArguments(args);
			return f;
		}

		@Override
		public Dialog onCreateDialog(Bundle savedInstanceState) {
			final Bundle args = getArguments();
			final int position = args.getInt("position");
			final String oldVal = args.getString("value");
			final View v = getActivity().getLayoutInflater().inflate(R.layout.dialog_label, null);
			((EditText) v.findViewById(R.id.edtLabel)).setText(oldVal);
			return new AlertDialog.Builder(getActivity())
				.setTitle("Label")
				.setView(v)
				.setPositiveButton("OK", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						final String newVal = ((EditText) EditLabelDialogFragment.this.getDialog().findViewById(R.id.edtLabel))
								.getText().toString();
						if (!newVal.equals(oldVal)) {
							((StorageSettingsFragment) getTargetFragment()).onDialogLabelChange(position, newVal);
						}
					}
				})
				.setNegativeButton("Cancel", new DialogInterface.OnClickListener() {
					@Override
					public void onClick(DialogInterface dialog, int which) {
						EditLabelDialogFragment.this.getDialog().cancel();
					}
				})
				.create();
		}
	}

	// Called on the exit from the dialog, if the label has changed.
	private void onDialogLabelChange(int position, String label) {
		mAdapter.getItem(position).writeLabelFile(label);
		refreshVolumes();
	}

	// There has been an addition or a subtraction of a Volume, request an updated list.
	private void renewVolumes() {
		mAdapter.clear();
		mAdapter.addAll(mVolumeManager.getVolumes());
		refreshVolumes();
	}

	// There has been a change in the properties of at least one of the items, request an updated display.
	private void refreshVolumes() {
		mAdapter.notifyDataSetChanged();
	}
}
